@@ -25,7 +25,7 @@
 #define NETWORKMANAGERQT_DEVICE_H
 
 #include <QObject>
-#include <QtCore/QtGlobal>
+#include <QtGlobal>
 #include <QSharedPointer>
 
 #include <networkmanagerqt/networkmanagerqt_export.h>
@@ -34,6 +34,7 @@
 #include "dhcp4config.h"
 #include "dhcp6config.h"
 #include "activeconnection.h"
+#include "devicestatistics.h"
 
 namespace NetworkManager
 {
@@ -65,9 +66,8 @@ class NETWORKMANAGERQT_EXPORT Device : public QObject
     Q_PROPERTY(bool firmwareMissing READ firmwareMissing)
     Q_PROPERTY(bool autoconnect READ autoconnect WRITE setAutoconnect)
     Q_PROPERTY(DeviceStateReason stateReason READ stateReason)
-
-    //Q_PROPERTY(Solid::Control::IPv4Config ipV4Config READ ipV4Config WRITE setIpV4Config)
     Q_PROPERTY(State state READ state)
+    Q_PROPERTY(NetworkManager::DeviceStatistics::Ptr deviceStatistics READ deviceStatistics)
 
 public:
     typedef QSharedPointer<Device> Ptr;
@@ -120,6 +120,7 @@ public:
                              NewActivation = 60, ParentChanged = 61, ParentManagedChanged = 62,
                              Reserved = 65536
                            };
+    Q_ENUM(StateChangeReason)
 
     enum MeteredStatus {
         UnknownStatus = 0,  /**< The device metered status is unknown. */
@@ -128,6 +129,7 @@ public:
         GuessYes = 3,  /**< The device is metered and the value was guessed. */
         GuessNo = 4  /**< The device is not metered and the value was guessed. */
     };
+    Q_ENUM(MeteredStatus)
 
     /**
      * Possible device capabilities
@@ -144,26 +146,38 @@ public:
      * Device type
      */
     enum Type {
-        UnknownType = NM_DEVICE_TYPE_UNKNOWN, /**< Unknown device type */
-        Ethernet = NM_DEVICE_TYPE_ETHERNET, /**< Ieee8023 wired ethernet */
-        Wifi = NM_DEVICE_TYPE_WIFI, /**< the Ieee80211 family of wireless networks */
-        Unused1 = NM_DEVICE_TYPE_UNUSED1, /**< Currently unused */
-        Unused2 = NM_DEVICE_TYPE_UNUSED2, /**< Currently unused */
-        Bluetooth = NM_DEVICE_TYPE_BT, /**< network bluetooth device (usually a cell phone) */
-        OlpcMesh = NM_DEVICE_TYPE_OLPC_MESH, /**< OLPC Mesh networking device */
-        Wimax = NM_DEVICE_TYPE_WIMAX, /**< WiMax WWAN technology */
-        Modem = NM_DEVICE_TYPE_MODEM, /**< POTS, GSM, CDMA or LTE modems */
+        UnknownType = NM_DEVICE_TYPE_UNKNOWN,   /**< Unknown device type */
+        Ethernet = NM_DEVICE_TYPE_ETHERNET,     /**< Ieee8023 wired ethernet */
+        Wifi = NM_DEVICE_TYPE_WIFI,             /**< the Ieee80211 family of wireless networks */
+        Unused1 = NM_DEVICE_TYPE_UNUSED1,       /**< Currently unused */
+        Unused2 = NM_DEVICE_TYPE_UNUSED2,       /**< Currently unused */
+        Bluetooth = NM_DEVICE_TYPE_BT,          /**< network bluetooth device (usually a cell phone) */
+        OlpcMesh = NM_DEVICE_TYPE_OLPC_MESH,    /**< OLPC Mesh networking device */
+        Wimax = NM_DEVICE_TYPE_WIMAX,           /**< WiMax WWAN technology */
+        Modem = NM_DEVICE_TYPE_MODEM,           /**< POTS, GSM, CDMA or LTE modems */
         InfiniBand = NM_DEVICE_TYPE_INFINIBAND, /**< Infiniband network device */
-        Bond = NM_DEVICE_TYPE_BOND, /**< Bond virtual device */
-        Vlan = NM_DEVICE_TYPE_VLAN, /**< Vlan virtual device */
-        Adsl = NM_DEVICE_TYPE_ADSL, /**< ADSL modem device */
-        Bridge = NM_DEVICE_TYPE_BRIDGE, /**< Bridge virtual device */
-        Generic = NM_DEVICE_TYPE_GENERIC, /**< Generic device @since 0.9.9.0 */
-        Team = NM_DEVICE_TYPE_TEAM, /**< Team master device @since 0.9.9.0 */
-        Gre, /**< Gre virtual device @since 0.9.9.0 */
-        MacVlan, /**< MacVlan virtual device @since 0.9.9.0 */
-        Tun, /**< Tun virtual device @since 0.9.9.0 */
-        Veth, /**< Veth virtual device @since 0.9.9.0 */
+        Bond = NM_DEVICE_TYPE_BOND,             /**< Bond virtual device */
+        Vlan = NM_DEVICE_TYPE_VLAN,             /**< Vlan virtual device */
+        Adsl = NM_DEVICE_TYPE_ADSL,             /**< ADSL modem device */
+        Bridge = NM_DEVICE_TYPE_BRIDGE,         /**< Bridge virtual device */
+        Generic = NM_DEVICE_TYPE_GENERIC,       /**< Generic device @since 1.0.0 */
+        Team = NM_DEVICE_TYPE_TEAM,             /**< Team master device @since 1.0.0 */
+        Gre,                                    /**< Gre virtual device @since 1.2.0, @deprecated use IpTunnel instead*/
+        MacVlan,                                /**< MacVlan virtual device @since 1.2.0 */
+        Tun,                                    /**< Tun virtual device @since 1.2.0 */
+        Veth,                                   /**< Veth virtual device @since 1.2.0 */
+        IpTunnel,                               /**< IP Tunneling Device @since 1.2.0 */
+        VxLan,                                  /**< Vxlan Device @since 1.2.0 */
+        MacSec,                                 /**< MacSec Device @since 1.6.0 */
+        Dummy,                                  /**< Dummy Device @since 1.8.0 */
+        Ppp,                                    /**< Ppp Device @since 1.10 */
+        OvsInterface,                           /**< OvsInterface Device @since 1.10 */
+        OvsPort,                                /**< OvsPort Device @since 1.10 */
+        OvsBridge,                              /**< OvsBridge Device @since 1.10 */
+        Wpan,                                   /**< Wpan Device @since 1.14 */
+        Lowpan,                                 /**< Lowpan Device @since 1.14 */
+        WireGuard,                              /**< WireGuard Device @since 1.14 */
+        WifiP2P                                 /**< WifiP2P Device @since 1.16 */
     };
     Q_ENUM(Type)
     Q_DECLARE_FLAGS(Types, Type)
@@ -174,7 +188,7 @@ public:
      *
      * @param path UNI of the device
      */
-    explicit Device(const QString &path, QObject *parent = 0);
+    explicit Device(const QString &path, QObject *parent = nullptr);
     Device(DevicePrivate &dd, QObject *parent);
     /**
      * Destroys a device object.
@@ -240,7 +254,6 @@ public:
      *
      * @since 5.8.0
      *
-     * @note this is a noop in runtime NM < 1.0.0
      */
     QDBusPendingReply<> deleteInterface();
     /**
@@ -348,7 +361,6 @@ public:
      * The device MTU (maximum transmission unit)
      * @since 0.9.9.0
      *
-     * @note always returns 0 in runtime NM < 1.0.6
      */
     uint mtu() const;
 
@@ -356,7 +368,6 @@ public:
      * @return If TRUE, indicates the NetworkManager plugin for the device is likely
      * missing or misconfigured.
      * @since 5.14.0
-     * @note always returns false in runtime NM < 1.2.0
      */
     bool nmPluginMissing() const;
 
@@ -364,7 +375,6 @@ public:
      * @return Whether the amount of traffic flowing through the device is
      * subject to limitations, for example set by service providers.
      * @since 5.14.0
-     * @note always returns UnknownStatus in runtime NM < 1.0.6
      */
     MeteredStatus metered() const;
 
@@ -376,6 +386,11 @@ public:
      */
     void setAutoconnect(bool autoconnect);
 
+    /**
+     * Returns Device Statistics interface
+     */
+    DeviceStatistics::Ptr deviceStatistics() const;
+
     /**
       * Retrieves a specialized interface to interact with the device corresponding
       * to a given device interface.
@@ -518,7 +533,6 @@ Q_SIGNALS:
      * Emitted when NmPluginMissing property has changed
      * @since 5.14.0
      * @see nmPluginMissing
-     * @note never emmited in runtime NM < 1.2.0
      */
     void nmPluginMissingChanged(bool nmPluginMissing);
 
@@ -526,7 +540,6 @@ Q_SIGNALS:
      * Emitted when metered property has changed
      * @since 5.14.0
      * @see metered
-     * @note never emmited in runtime NM < 1.0.6
      */
     void meteredChanged(MeteredStatus metered);
 
@@ -574,4 +587,3 @@ private:
 }
 
 #endif
-
